if RequiredScript == "lib/managers/menumanager" then
	if not _G.Replenished then
		_G.Replenished = _G.Replenished or {}
		Replenished.DownsCounter = {0,0,0,0}
		Replenished.Name = "Replenished"
		Replenished.Toggle = true
		Replenished.Colors = {"ff0000", "00ff04", "1a64f6"} -- RGB
		Replenished.Down_Warn = 3
	end
	function Replenished:Health(peer_id)
		local peer = managers.network:session():peer(peer_id)
		if peer then
			local down
			if Replenished.DownsCounter[peer_id] > 1 or Replenished.DownsCounter[peer_id] == 0 then
				down = "downs"
			elseif Replenished.DownsCounter[peer_id] == 1 then
				down = "down"
			end
			local message = peer:name() .. " replenished +" .. tostring(Replenished.DownsCounter[peer_id]) .. " " .. down .. "."
			Replenished:DoPrint(message, Replenished.Colors[2])
			Replenished.DownsCounter[peer_id] = 0
		end
	end

	function Replenished:Downed(peer_id, local_peer)
		local peer = managers.network:session():peer(peer_id)
		if peer then
			local Need_Downs = Replenished.Down_Warn
			if local_peer then
				local nine_lives = managers.player:upgrade_value( 'player', 'additional_lives', 0) or 0
				Need_Downs = Need_Downs + nine_lives
			end
			Replenished.DownsCounter[peer_id] = Replenished.DownsCounter[peer_id] + 1
			if Replenished.DownsCounter[peer_id] == Need_Downs then
				local message = "Warning! " .. peer:name() .. " was downed " .. tostring(Replenished.DownsCounter[peer_id]) .. " times."
				Replenished:DoPrint(message, Replenished.Colors[1])
			end
		end
	end
	
	function Replenished:Peer_id_To_Peer(peer_id)
		local session = managers.network:session()
		return session and session:peer(peer_id)
	end
	
	function Replenished:_peer(input)
		local t = type(input)
		if t == 'userdata' then
			return alive(input) and input:network():peer()
		end
		if t == 'number' then
			return self:Peer_id_To_Peer(input)
		end
		if t == 'string' then
			return self:_peer(managers.criminals:character_peer_id_by_name(input))
		end
	end
	
	function Replenished:_peer_info(input)
		local peer = self:_peer(input)
		return peer and peer:id() or 0
	end
	
	function Replenished:EnterCustody(criminal_name, local_peer)
		local peer_id
		if not local_peer then
			for __, data in pairs(managers.criminals._characters) do
				if data.taken and criminal_name == data.name then
					peer_id = data.peer_id
					break
				end
			end
		else
			peer_id = criminal_name
		end
		local peer = managers.network:session():peer(peer_id)
		if peer then
			Replenished.DownsCounter[peer_id]=0
			--local message = peer:name() .. " went to custody."
			--Replenished:DoPrint(message, Replenished.Colors[1])
		end
	end
	
	function Replenished:DoPrint(message, state)
		if Replenished.Toggle == true then
			managers.chat:_receive_message(1, Replenished.Name, message, Color(state))
			for i=1,4 do
				if _G.LuaNetworking:LocalPeerID() ~= i then
					local peer2 = managers.network:session() and managers.network:session():peer(i)
					if peer2 then
						peer2:send("send_chat_message", ChatManager.GAME, Replenished.Name .. ": " .. message)
					end
				end
			end
		end
	end
end

if RequiredScript == "lib/units/equipment/doctor_bag/doctorbagbase" then
	Hooks:PostHook(DoctorBagBase, "take", "Replenished:MedBag", function(self, unit)
		Replenished:Health(_G.LuaNetworking:LocalPeerID()) -- i took meds
	end)
end

if RequiredScript == "lib/units/beings/player/huskplayermovement" then
	Hooks:PostHook(HuskPlayerMovement, '_start_bleedout', "Replenished:DownedOther", function( self, ... )
		local peer_id = Replenished:_peer_info(self._unit)
		if peer_id then
			Replenished:Downed(peer_id) -- other was downed
		end
	end)
end

if RequiredScript == "lib/units/beings/player/states/playerbleedout" then
	Hooks:PostHook(PlayerBleedOut, '_enter', "Replenished:Downed", function(self, enter_data)
		Replenished:Downed(_G.LuaNetworking:LocalPeerID(), true) -- i was downed
	end)
end

if RequiredScript == "lib/network/handlers/unitnetworkhandler" then
	Hooks:PostHook(UnitNetworkHandler, "sync_doctor_bag_taken", "Replenished:MedBagOther", function(self, unit, amount, sender)
		local peer = self._verify_sender(sender)
		local peer_id = peer and peer:id()
		if peer_id then
			Replenished:Health(peer_id) --other took meds
		end
	end)
	
	Hooks:PostHook(UnitNetworkHandler, 'set_trade_death', "Replenished:CustodyOther", function(self, criminal_name, respawn_penalty, hostages_killed)
		Replenished:EnterCustody(criminal_name) --other went to custody
	end)
end

if RequiredScript == "lib/network/base/networkpeer" then
	Hooks:Add("BaseNetworkSessionOnPeerRemoved", "Replenished:PeerRemoved", function(peer, peer_id, reason)
		Replenished.DownsCounter[peer_id]=0 -- peer reset to zero
		Replenished.Toggle = true
	end)
end

if RequiredScript == "lib/managers/trademanager" then
	Hooks:PostHook(TradeManager, 'on_player_criminal_death', "Replenished:Custody", function(self, criminal_name, respawn_penalty, hostages_killed, skip_netsend)
		Replenished:EnterCustody(_G.LuaNetworking:LocalPeerID(), true) --local peer custody
	end)
end

if RequiredScript == "lib/managers/chatmanager" then
	Hooks:PostHook(ChatManager, "_receive_message", "Replenished:MessageReceived", function(self, channel_id, name, message)
		if Utils:IsInHeist() then
		local peer_id = ""
			for _, peer in pairs(managers.network:session():peers()) do
				if peer:name() == name then
					peer_id = peer:id()
					break
				end
			end
			if peer_id ~= "" then
				if string.sub(message, 1, string.len(Replenished.Name)) == Replenished.Name then
					managers.chat:_receive_message(1, Replenished.Name, "Same string", Color(state))
					if Replenished.Toggle == true then
						if peer_id < _G.LuaNetworking:LocalPeerID() then
							Replenished.Toggle = false
							managers.chat:_receive_message(1, Replenished.Name, "Mod disabled", Color(Replenished.Colors[3]))
						end
					end
				end
			end
		end
	end)
end